module AuthJwt
  # Include #login_user and #require_auth and @current_user to all Controllers.
  module ControllerAdditions
    # Add InstanceMethods
    def self.included(klass)
      klass.class_eval do
        include InstanceMethods
      end
    end

    # InstanceMethods to be included
    module InstanceMethods
      # The current user or nil if not authenticated
      attr_reader :current_user

      # Sets up a method to check a user credentials
      #
      # return the authenticated user or raise AuthJwt::Unauthorized
      #
      #
      # ```ruby
      #   class AnyController < ApplicationController
      #     def login
      #       login_user(credentials)
      #     end
      #   end
      # ```
      def login_user(credentials)
        user_class = AuthJwt.configuration.user_class.constantize
        login_field = AuthJwt.configuration.login_field.to_sym
        password_field = AuthJwt.configuration.password_field.to_sym

        user = user_class.find_by(login_field => credentials[login_field])
        if user
          fail AuthJwt::Unauthorized, 'Invalid credentials' unless user.authenticate(credentials[password_field])
          @current_user = user
        else
          fail AuthJwt::Unauthorized, 'Invalid user'
        end
      end

      # Sets up a method to check if the user is authenticated
      #
      # * if the user is authenticated setup current_user
      # * if the user is not authenticated, raise AuthJwt::Unauthorized
      #
      # ```ruby
      #   class AnyController < ApplicationController
      #     before_filter :require_auth
      #   end
      # ```
      def require_auth
        fail 'not in a controller scope' if request.nil?
        fail AuthJwt::Unauthorized, 'No Auth' if request.authorization.nil?
        user_class = AuthJwt.configuration.user_class.constantize
        begin
          user = user_class.from_jwe request.authorization
          if user
            @current_user = user
          else
            fail 'Not Found'
          end
        rescue
          raise AuthJwt::Unauthorized, 'Invalid token'
        end
      end
    end
  end
end
